@@ -1,3 +1,7 @@
+#ifndef _PYPY_PYPY_DECL_H
+
+#define _PYPY_PYPY_DECL_H
+
 #ifndef PYPY_STANDALONE
 
 #ifdef __cplusplus
@@ -106,6 +110,7 @@ PyAPI_FUNC(void) PyErr_Restore(PyObject *arg0, PyObject *arg1, PyObject *arg2);
 PyAPI_FUNC(void) PyErr_SetExcInfo(PyObject *arg0, PyObject *arg1, PyObject *arg2);
 PyAPI_FUNC(PyObject *) PyErr_SetFromErrno(PyObject *arg0);
 PyAPI_FUNC(PyObject *) PyErr_SetFromErrnoWithFilename(PyObject *arg0, char *arg1);
+PyAPI_FUNC(PyObject *) PyErr_SetFromErrnoWithFilenameObject(PyObject *arg0, PyObject *arg1);
 PyAPI_FUNC(void) PyErr_SetInterrupt(void);
 PyAPI_FUNC(void) PyErr_SetNone(PyObject *arg0);
 PyAPI_FUNC(void) PyErr_SetObject(PyObject *arg0, PyObject *arg1);
@@ -147,8 +152,8 @@ PyAPI_FUNC(PyFrameObject *) PyFrame_New(PyThreadState *arg0, PyCodeObject *arg1,
 PyAPI_FUNC(int) PyFunction_Check(PyObject *arg0);
 PyAPI_FUNC(int) PyFunction_CheckExact(PyObject *arg0);
 PyAPI_FUNC(PyObject *) PyFunction_GetCode(PyObject *arg0);
-PyAPI_FUNC(PyGILState_STATE) PyGILState_Ensure(void);
-PyAPI_FUNC(void) PyGILState_Release(PyGILState_STATE arg0);
+PyAPI_FUNC(int) PyGILState_Ensure(void);
+PyAPI_FUNC(void) PyGILState_Release(int arg0);
 PyAPI_FUNC(PyObject *) PyImport_AddModule(const char *arg0);
 PyAPI_FUNC(PyObject *) PyImport_ExecCodeModule(char *arg0, PyObject *arg1);
 PyAPI_FUNC(PyObject *) PyImport_ExecCodeModuleEx(char *arg0, PyObject *arg1, char *arg2);
@@ -546,13 +551,13 @@ PyAPI_DATA(PyObject*) PyExc_Exception;
 PyAPI_DATA(PyObject*) PyExc_TypeError;
 PyAPI_DATA(PyObject*) PyExc_SystemError;
 PyAPI_DATA(PyObject*) PyExc_ReferenceError;
-PyAPI_DATA(PyTypeObject) PyNotImplemented_Type;
+PyAPI_DATA(PyTypeObject) PyNone_Type;
 PyAPI_DATA(PyTypeObject) PySet_Type;
 PyAPI_DATA(PyObject*) PyExc_TabError;
 PyAPI_DATA(PyTypeObject) PyDict_Type;
 PyAPI_DATA(PyTypeObject) PyByteArray_Type;
-PyAPI_DATA(PyTypeObject) PyNone_Type;
-PyAPI_DATA(PyTypeObject) PyLong_Type;
+PyAPI_DATA(PyTypeObject) PyList_Type;
+PyAPI_DATA(PyTypeObject) PyBaseString_Type;
 PyAPI_DATA(PyTypeObject) PyWrapperDescr_Type;
 PyAPI_DATA(PyObject*) PyExc_PendingDeprecationWarning;
 PyAPI_DATA(PyObject*) PyExc_OverflowError;
@@ -568,17 +573,17 @@ PyAPI_DATA(PyObject*) PyExc_FutureWarning;
 PyAPI_DATA(PyObject) _Py_ZeroStruct;
 PyAPI_DATA(PyObject*) PyExc_UnboundLocalError;
 PyAPI_DATA(PyObject) _Py_NotImplementedStruct;
-PyAPI_DATA(PyTypeObject) PyList_Type;
 PyAPI_DATA(PyTypeObject) PyComplex_Type;
-PyAPI_DATA(PyTypeObject) PyFrozenSet_Type;
+PyAPI_DATA(PyTypeObject) PyFloat_Type;
 PyAPI_DATA(PyTypeObject) PyUnicode_Type;
 PyAPI_DATA(PyTypeObject) PyCFunction_Type;
+PyAPI_DATA(PyTypeObject) PyArray_Type;
 PyAPI_DATA(PyObject*) PyExc_BytesWarning;
 PyAPI_DATA(PyObject*) PyExc_DeprecationWarning;
 PyAPI_DATA(PyObject*) PyExc_SyntaxError;
 PyAPI_DATA(PyObject*) PyExc_UnicodeWarning;
 PyAPI_DATA(PyObject*) PyExc_ZeroDivisionError;
-PyAPI_DATA(PyTypeObject) PyFloat_Type;
+PyAPI_DATA(PyTypeObject) PyFrozenSet_Type;
 PyAPI_DATA(PyObject*) PyExc_RuntimeWarning;
 PyAPI_DATA(PyObject) _Py_NoneStruct;
 PyAPI_DATA(PyObject*) PyExc_IndentationError;
@@ -587,12 +592,12 @@ PyAPI_DATA(PyObject*) PyExc_GeneratorExit;
 PyAPI_DATA(PyObject*) PyExc_ImportWarning;
 PyAPI_DATA(PyObject*) PyExc_UnicodeEncodeError;
 PyAPI_DATA(PyTypeObject) PyInt_Type;
+PyAPI_DATA(PyTypeObject) PyNotImplemented_Type;
 PyAPI_DATA(PyTypeObject) PyString_Type;
 PyAPI_DATA(PyTypeObject) PyBool_Type;
 PyAPI_DATA(PyObject*) PyExc_OSError;
 PyAPI_DATA(PyObject*) PyExc_KeyError;
 PyAPI_DATA(PyObject*) PyExc_SyntaxWarning;
-PyAPI_DATA(PyTypeObject) PyBaseString_Type;
 PyAPI_DATA(PyObject*) PyExc_StopIteration;
 PyAPI_DATA(PyObject*) PyExc_NotImplementedError;
 PyAPI_DATA(PyObject*) PyExc_ImportError;
@@ -604,6 +609,7 @@ PyAPI_DATA(PyTypeObject) PyClass_Type;
 PyAPI_DATA(PyTypeObject) PyType_Type;
 PyAPI_DATA(PyTypeObject) PyMemoryView_Type;
 PyAPI_DATA(PyObject*) PyExc_UnicodeTranslateError;
+PyAPI_DATA(PyTypeObject) PyLong_Type;
 PyAPI_DATA(PyObject*) PyExc_LookupError;
 PyAPI_DATA(PyObject*) PyExc_EOFError;
 PyAPI_DATA(PyObject*) PyExc_BufferError;
@@ -618,3 +624,5 @@ PyAPI_DATA(PyObject*) PyExc_KeyboardInterrupt;
 }
 #endif
 #endif /*PYPY_STANDALONE*/
+
+#endif /*_PYPY_PYPY_DECL_H*/
